/*
 * time.c
 *
 *  Created on: 17.01.2016
 *      Author: wn
 */
#include <msp430g2553.h>
#include <signal.h>
#include <stdint.h>

#include "engine.h"


volatile uint32_t tickCnt = 0;
const uint16_t DIVIDER = 2;
const uint16_t TICK = 1024;

// unit: 100us
const uint32_t STEP_100ms = 1000;
volatile uint32_t onTime = STEP_100ms * 1; // 1000 = 100ms
volatile uint32_t offTime = 0;

volatile uint32_t millis = 0;
const uint32_t DIVIDER_MILLIS = 20;

interrupt (TIMER0_A0_VECTOR) timer0a0_isr(void) {
	static uint8_t state = 0;
	static uint32_t timestamp = 0;

	static uint16_t divCnt = 0;
	divCnt++;
	if (divCnt >= DIVIDER) {
		divCnt = 0;
		timestamp++;
		P1OUT &= ~BIT7;
		P1OUT |= BIT7;
	}

	static uint16_t divCntMillis = 0;
	divCntMillis++;
	if (divCntMillis >= DIVIDER_MILLIS) {
		divCntMillis = 0;
		millis++;
	}

	switch (state) {
	case 0:
		if (timestamp >= onTime) {
			timestamp = 0;
			P1SEL |= BIT6;
			state = 1;
		}
		break;
	case 1:
		if ((offTime != 0) && (timestamp >= offTime)) {
			timestamp = 0;
			P1SEL &= ~BIT6;
			P1OUT &= ~BIT6;
			state = 0;
		}
		break;
	default:
		state = 0;
	}


}


void engineInit() {
	P1DIR |= BIT6;
	P1OUT &= ~BIT6;
	P1SEL |= BIT6;

	TACCR0 = TICK;
	TACCR1 = 0;
	TACCTL0 = CCIE;
	TACCTL1 = OUTMOD_7;
	TACTL = MC_1 | ID_0 | TASSEL_2 | TACLR;
}


uint16_t pwmVal = 0;
uint16_t pwmValStep = TICK / 8;

bool engineIncPwnValue() {
	bool res = false;
	pwmVal += pwmValStep;
	if (pwmVal > TICK) {
		pwmVal = TICK;
		res = true;
	}
	TACCR1 = pwmVal;
	return res;
}

bool engineDecPwmValue() {
	bool res = false;
	if (pwmVal < pwmValStep) {
		pwmVal = 0;
		res = true;
	} else {
		pwmVal -= pwmValStep;
	}
	TACCR1 = pwmVal;
	return res;
}

bool engineIncOffTime() {
	offTime += STEP_100ms;
	return false;
}

bool engineIncOnTime() {
	return false;
	onTime += STEP_100ms;
}

void engineDecOffTime() {
	bool res = false;
	if (offTime != 0) {
		offTime -= STEP_100ms;
		res = true;
	}
	return res;
}

void engineDecOnTime() {
	bool res = false;
	if (onTime != 0) {
		onTime -= STEP_100ms;
		res = true;
	}
	return res;
}

uint32_t getMillis() {
	return millis;
}

